-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
local logTag = "activeStrut"
M.type = "auxilliary"
M.relevantDevice = nil
M.defaultOrder = 1000


local psiToPascal = 6894.76

local airbagController = nil
local minLevel = 20
local midLevel = 55
local maxLevel = 100
local currentLevel = 0
local currentPressure = 0
local rollCompensate = 0.15
local frontDynamicRatio = 0.7
local rearDynamicRatio = 0.7

local counter = 0

local wheels = {} --cache for holding all our wheels
local frontLeftWheelName = ""
local frontRightWheelName = ""
local rearRightWheelName = ""
local rearLeftWheelName = ""

local function controlVer1()

--local lateralG = gxSmooth:get(sensors.gx2)
	local steer = electrics.values['steering_input']
	local wheelspeed = electrics.values.wheelspeed
	
	if steer < -rollCompensate and wheelspeed > 10 then 
		--airbagController.setBeamPressureLevel({'rightAxle'}, currentLevel *( math.abs(steer) * rollCompensate + 1))
		--airbagController.setBeamPressureLevel({'leftAxle'}, currentLevel)
		
		airbagController.setBeamMin({'leftAxle'});
		airbagController.setBeamMax({'rightAxle'});
		
	end
	
	if steer > rollCompensate and wheelspeed > 10 then 
		--airbagController.setBeamPressureLevel({'leftAxle'}, currentLevel * (math.abs(steer) * rollCompensate + 1))
		--airbagController.setBeamPressureLevel({'rightAxle'}, currentLevel)
		
		airbagController.setBeamMin({'rightAxle'});
		airbagController.setBeamMax({'leftAxle'});
	end
	
	if steer >= -rollCompensate and steer <= rollCompensate then
		if currentLevel == -1 then
			airbagController.setBeamMin({'leftAxle'})
			airbagController.setBeamMin({'rightAxle'})
		end
		if currentLevel == 0 then
			airbagController.setBeamDefault({'leftAxle'})
			airbagController.setBeamDefault({'rightAxle'})
		end
		if currentLevel == 1 then
			airbagController.setBeamMax({'leftAxle'})
			airbagController.setBeamMax({'rightAxle'})
		end
	end

end

local function controlVer2()

	local lateralG = math.min(math.pow(math.abs(gxSmooth:get(sensors.gx2)), 0.5), 1.0)
	local longitudinalG = math.min(math.pow(math.abs(gySmooth:get(sensors.gy2)), 0.3), 1.0)
	local wheelLoads = {}
	local totalwheelLoads = 0
	
	  for _,w in pairs(powertrain.wheels) do
		local wheelLoad = w.downForce
		wheelLoads[w.name] = wheelLoad
		totalwheelLoads = wheelLoad + totalwheelLoads
		--log('I', "Wheel Name ", w.name)
	  end
	  
	local averageWheelLoad = totalwheelLoads / 4
	  
	if currentLevel == -1 then
			currentPressure = minLevel
		end
		if currentLevel == 0 then
			currentPressure = midLevel
		end
		if currentLevel == 1 then
			currentPressure = maxLevel
		end
	  
	if counter == 100 then

	local flantiRollForce = (wheelLoads[frontLeftWheelName] - averageWheelLoad) * lateralG
	local frantiRollForce = (wheelLoads[frontRightWheelName] - averageWheelLoad) * lateralG
	local rlantiRollForce = (wheelLoads[rearLeftWheelName] - averageWheelLoad) * lateralG
	local rrantiRollForce = (wheelLoads[rearRightWheelName] - averageWheelLoad) * lateralG 
	
	local flantiPitchForce = math.max((wheelLoads[frontLeftWheelName] - averageWheelLoad), 0) * longitudinalG * (1 - lateralG)
	local frantiPitchForce = math.max((wheelLoads[frontRightWheelName] - averageWheelLoad), 0) * longitudinalG * (1 - lateralG)
	local rlantiPitchForce = math.max((wheelLoads[rearLeftWheelName] - averageWheelLoad), 0) * longitudinalG * (1 - lateralG)
	local rrantiPitchForce = math.max((wheelLoads[rearRightWheelName] - averageWheelLoad),0) * longitudinalG * (1 - lateralG)
	
	local flantiBumpForce = math.max((wheelLoads[rearLeftWheelName] - wheelLoads[frontLeftWheelName]), 0) * (1 - longitudinalG) * (1 - lateralG)
	local frantiBumpForce = math.max((wheelLoads[rearRightWheelName] - wheelLoads[frontRightWheelName]), 0) * (1 - longitudinalG) * (1 - lateralG)
	local rlantiBumpForce = math.max((wheelLoads[frontLeftWheelName] - wheelLoads[rearLeftWheelName]), 0) * (1 - longitudinalG) * (1 - lateralG)
	local rrantiBumpForce = math.max((wheelLoads[frontRightWheelName] - wheelLoads[rearRightWheelName]), 0) * (1 - longitudinalG) * (1 - lateralG)
	
	local flFinalForce = flantiRollForce + flantiPitchForce
	local frFinalForce = frantiRollForce + frantiPitchForce
	local rlFinalForce = rlantiRollForce + rlantiPitchForce
	local rrFinalForce = rrantiRollForce + rrantiPitchForce

	if lateralG > 0.1 and longitudinalG > 0.3 then
	airbagController.setBeamPressure({'fleftAxle'}, math.max(flFinalForce / 0.0097 / psiToPascal / frontDynamicRatio + currentPressure ), 0)
	end
	
	if lateralG > 0.1 and longitudinalG > 0.3 then
	airbagController.setBeamPressure({'frightAxle'}, math.max(frFinalForce / 0.097 / psiToPascal / frontDynamicRatio + currentPressure ), 0)
	end
	
	if lateralG > 0.1 and longitudinalG > 0.3 then
	airbagController.setBeamPressure({'rleftAxle'}, math.max(rlFinalForce / 0.006 / psiToPascal / rearDynamicRatio + currentPressure), 0)
	end
	
	if lateralG > 0.1 and longitudinalG > 0.3 then
	airbagController.setBeamPressure({'rrightAxle'}, math.max(rrFinalForce / 0.006 / psiToPascal / rearDynamicRatio + currentPressure), 0)
	end
	
	counter = 0
	
	end
	
	counter = counter + 1
	
end

local function updateWheelsIntermediate()

	if airbagController == nil then return end

	controlVer2()
	
end




local function setMin()
  

	currentLevel = -1
	airbagController.setBeamPressure({'fleftAxle', 'frightAxle', 'rleftAxle', 'rrightAxle'}, minLevel)
	
end

local function setMid()
  

  currentLevel = 0
  airbagController.setBeamPressure({'fleftAxle', 'frightAxle', 'rleftAxle', 'rrightAxle'}, midLevel)

end

local function setMax()
  

 currentLevel = 1
 airbagController.setBeamPressure({'fleftAxle', 'frightAxle', 'rleftAxle', 'rrightAxle'}, maxLevel)
 
end


local function updateGFX(dt)
 

  
  
end

local function reset()
  M.level = 0
  
end



local function init(jbeamData)
  
  

  M.level = 0
  currentLevel = 0
  currentPressure = midLevel
 

  
end

local function initSecondStage()

  local avgWheelPos = vec3(0,0,0)
  local wheelCount = 0
  wheels = {}
  
  for _,wd in pairs(powertrain.wheels) do
    local name = wd.name
    wheels[name] = wd
    --wheels[name].speedSmoother =  newExponentialSmoothing(500)
    --log('I', "Wheel Name ", name)
  end
  
  
  for _,w in pairs(powertrain.wheels) do
    
    local wheelNodePos = v.data.nodes[wheels[w.name].node1].pos --find the wheel position
    avgWheelPos = avgWheelPos + wheelNodePos --sum up all positions
    wheelCount = wheelCount + 1
  end
  
  local vectorForward = vec3(v.data.nodes[v.data.refNodes[0].ref].pos) - vec3(v.data.nodes[v.data.refNodes[0].back].pos) -- vec3(obj:getDirectionVector()) --vector facing forward
  local vectorUp = vec3(v.data.nodes[v.data.refNodes[0].up].pos) - vec3(v.data.nodes[v.data.refNodes[0].ref].pos)

  local vectorRight = vectorForward:cross(vectorUp) --vector facing to the right

  --iterate over all wheels 
  for _,w in pairs(powertrain.wheels) do
    local wheelNodePos = vec3(v.data.nodes[wheels[w.name].node1].pos) --find the wheel position
    local wheelVector = wheelNodePos - avgWheelPos --create a vector from our "center" to the wheel
    local dotForward = vectorForward:dot(wheelVector) --calculate dot product of said vector and forward vector
    local dotLeft = vectorRight:dot(wheelVector) --calculate dot product of said vector and left vector

    if dotForward >= 0 then
      if dotLeft >= 0 then
        
		frontRightWheelName = w.name --this case can only mean it's our front right wheel
      else
        frontLeftWheelName = w.name -- ...
      end
    else
      if dotLeft >= 0 then
        rearRightWheelName = w.name-- ...
      else
        rearLeftWheelName = w.name-- ...
      end
    end
  end



  
   airbagController = controller.getController("airbags")
   M.updateWheelsIntermediate = updateWheelsIntermediate
   M.updateGFX = updateGFX
   gxSmooth = newExponentialSmoothing(20)
   gySmooth = newExponentialSmoothing(15)
   
end







M.init = init
M.initSecondStage = initSecondStage
M.reset = reset
M.updateGFX = nil
M.updateWheelsIntermediate = nil
M.setMin = setMin
M.setMid = setMid
M.setMax = setMax




return M